/***************************************************************************

        065L_prom.asm

        PROM based boot loader of the ADSP21065L processor.

        Copyright (c) 1998 Analog Devices Inc.  All rights reserved.

		Modified: October-5-1999
				  June-28-2000	

		Changes Made: Per the 8-25-99 21065L Anomaly List, set the 
		              EIEPx bit A23 for PROM Booting
					  
					  Modified code to accomodate EPROM overlays - GJO 2000
***************************************************************************/
 	
/* Warning: After booting process, EP0I bit in IMASK is set.	*/

/* Define the addresses of various IOP registers.		*/
#include <def21065l.h>

/* ADSP-21065L specific reset vector start address.*/
#define START_ADDR 0x8000
#define START_DM	0xC000

.SEGMENT/PM     seg_ldr;

/* The loader begins with the interrupts up to and including the low 	*/
/* priority timer interrupt.											*/

                NOP;NOP;NOP;NOP;	/* Reserved interrupt           	*/

___lib_RSTI: 	IDLE;			/* Implicit IDLE instruction	*/
		JUMP Start_Loader (DB);	/* Begin loader					*/
		NOP;					/* Pad to next interrupt		*/
		NOP;					/* Pad to next interrupt		*/

                NOP;NOP;NOP;NOP;

/* Vector for status stack/loop stack overflow or PC stack full:        */
___lib_SOVFI:   RTI;                                                                        
                RTI;
                RTI;
                RTI;

/* Vector for high priority timer interrupt:                            */
___lib_TMZHI:   RTI;
                RTI;
                RTI;
                RTI;

/* Vector for external interrupts:                                      */
___lib_VIRPTI:  RTI;
                RTI;
                RTI;
                RTI;
                
___lib_IRQ2I:   RTI;
                RTI;
                RTI;
                RTI;

___lib_IRQ1I:   RTI;
                RTI;
                RTI;
                RTI;

___lib_IRQ0I:   RTI;
                RTI;
                RTI;
                RTI;

                NOP;NOP;NOP;NOP;

/* Vector for Serial port DMA channels:                                */

___lib_SPROI:   RTI;
                RTI;
                RTI;
                RTI;

___lib_SPR1I:   RTI;
                RTI;
                RTI;
                RTI;

___lib_SPT0I:   RTI;
                RTI;
                RTI;
                RTI;

___lib_SPT1I:   RTI;
                RTI;
                RTI;
                RTI;

/* Reserved */	RTI;
                RTI;
                RTI;
                RTI;

/* Reserved */	RTI;
                RTI;
                RTI;
                RTI;

/* Vectors for External port DMA channels: 				*/
___lib_EP0I:	R2=DM(DMAC0);		/* Get DMAC Control setting	*/
		RTI (DB);
		R6=0;
		DM(DMAC0)=R6;				/*  zeroed between uses.	*/

Start_Loader:
/* After power up or reset, default value in SYSCON register is 0x0000 0010.	*/
/* The WAIT register for revision 0.0 defaults to 0x200d6b5a. Rev 0.0 requires  */
/* that waitstates be inserted using external logic to assert the ACK pin (see  */
/* anomaly list).  For revisions 0.1 and greater, the WAIT register defaults    */ 
/* to 0x21ad6b5a, which correctly sets 6 waitstates plus an idle cycle for ROM  */
/* boot memory. If for any reason these two value should be modified, do it     */
/* here as following example. Make sure this file does not exceed 256 words.    */
/*                                                    			                */
/*		ustat1=0Xxxxxxxxx;													    */
/*		DM(SYSCON)=ustat1; 		Modify SYSCON								    */
/*		ustat1=0Xxxxxxxxx;													    */
/*		DM(WAIT)=ustat1;   		Modify WAIT									    */



/**************************************************************/
/* SETUP SDRAM on EZ-LAB */
/**************************************************************/
		ustat1=dm(WAIT);
		bit clr ustat1 0x000f801f;	/*clear MS0 amd MS3 waitstate and mode*/
		dm(WAIT)=ustat1;
		ustat1=937;					/*refresh rate*/
		dm(SDRDIV)=ustat1;
		ustat1=dm(IOCTL);			/*mask in SDRAM settings*/
		bit set ustat1 SDPSS|SDBN2|SDBS0|SDTRP3|SDTRAS5|SDCL2|SDPGS256|DSDCK1;
		dm(IOCTL)=ustat1;
/**************************************************************/
/**************************************************************/

	
		I2 = START_DM;	/* Used for eprom overlay */
	
		L0=0;			/* Zero out L-registers so they */
		L4=0;			/*  can be used without wrap	*/
		L7=0;
		L8=0;
		L12=0;
		L15=0;

		M5=0;			/* Setup M-registers to use for	*/			
		M6=1;			/*  for various memory transfers*/
		M13=0;
		M14=1;
				
		R10=DM(SYSCON);			/* Read current SYSCON setting	*/
		R12=PASS R10;			/* Hold Initial SYSCON setting	*/
		R11=BSET R10 BY 1;		/* Set BSO bit for reading ROM	*/
		R10=BCLR R10 BY 1;		/* Clear BSO bit for ext write	*/

		DM(SYSCON)=R10;			/* Clear BSO bit for writing RAM*/

		BIT SET IMASK 0x10000;	/* Enable EP0 interrupt	*/
		BIT SET MODE1 0x1800;	/* Enable interrupts and nesting*/

		R14=0x0221;             /* For DMAC0 setting            */

		R15=6;					/* EC to load one 48-bit word   */
		R0=DM(SYSTAT);			/* Load the Host ID             */
		R0=FEXT R0 BY 8:3;  
   
		DM(IMEP0)=M13;			/* Setup the DMA registers      */
		DM(EMEP0)=M14;

		I15 = START_ADDR+0x4;	/* DMA destination address      */

	/** kernel modification (next 3 instructions) 10/5/99 **/
		ustat1 = DM(EIEP0);      /* Get current index addr, points to 1st addr after last 256 kernel byte */
		bit set ustat1 0x800000; /* set bit A23 to ensure proper number of waitstates with BS0 set */
		DM(EIEP0) = ustat1;		 /* write back to the external DMA index register to read the next prom word */
	/** ************************************************* **/

get_addr:       CALL read_PROM_word;
                COMP(R0, R2);

                IF NE JUMP get_addr;

				/* ********************* Workaround for VDSP 4.1 elf-loader 8/25/99 ******************* */
				/*   Reset state of EIEP0 in PROM boot mode is different for early revisions */
				/*   test bits 25-29 in the MODE2 register to determine revision of the DSP */
				/*   Revision 0.0, 0.1 --> MODE2[31-25]=1100000  */
				/*   Revision 0.2      --> MODE2[31-25]=1100001  */
				/*   Revision 0.3      --> MODE2[31-25]=1100010  */

	    		R4=MODE2;				/* read MODE2 register to determine the DSP revision */
				R4=FEXT R4 BY 25:5;		/* extract bits 25-29, place into bits 0-4 back in same reg R4 */
				R4=PASS R4;				/* Pass throught the ALU to test the AZ bit in ASTAT for zero */
				IF EQ R3=BSET R3 BY 17;	/* If bits 25-20 are zero, the DSP is a rev 0.0 or 0.1 */
										/* Setting this is necessary for these revs since EIEP0 resets */
										/* to 0x20000. Setting bit 17 is not required if revision */
										/* is >= 0.2 because for these EIEP0 resets to 0x000000 */

				R3 = BSET R3 BY 23;		/* Setting A23 in external DMA index register is required  */
										/* for generating correct # of waitstates offchip with the BSO bit */
										/* (in SYSCON) set. Refer to Doc changes section in the 21065L anomaly list (8-25-99). */
										/* NOTE: A23 is set internally and is not driven out on an */
										/* external address pin, therefore it has no external affect */
										/* on the booting device */
				/* ************************************************************************************ */

                DM(EIEP0)=R3;             	/* Point to address in PROM */

read_boot_info: CALL read_PROM_word;
				R0=PASS R2;

                CALL read_PROM_word;

load_memory:	R0=PASS R0;
		IF NE JUMP (PC, test_dm16_zero);

/* After the IDLE completes, the following sequence of instructions will*/
/* be executed: (Remember these are in a loop)				*/
/*	R0=R0-R0, DM(I4,M5)=R9, PM(I12,M13)=R11				*/
/* This instructions sets the EQ flag to terminate the loop, writes 	*/
/* the original value to SYSCON, and writes a *new* instruction over	*/
/* itself.  The new instruction is:					*/
/*	PM(0, I8)=PX;							*/
/* This instruction resets the DMA8 vector to whatever it should be.	*/
/* The loop will terminate, because of the previous set EQ.  Instruction*/
/* flow will continue with START_ADDR+0x5, just like nothing happened!*/

final_init:	R9=0xb1db0000;	/* Load instruction PM(0,I8)=PX;*/
		R11=BSET R11 BY 8;	/* Set IMDW to 1 for inst write */             
		DM(SYSCON)=R11;		/* Setup to read PROM, inst wrt	*/

		R1=START_ADDR;		/* Point to destination		*/
		R2=0x100;			/* Load length of last init	*/
		R4=R2*R15 (SSI);	/* Compute external length	*/

		DM(IMEP0)=M14;		/* Set to increment internal ptr*/

		I4=START_ADDR+0x4;	/* Point to START_ADDR+0x4 for patch*/
		I8=START_ADDR+0x40;	/* Point to DMA8 vector to patch*/
		I12 = SYSCON;		/* Address of SYSCON		*/
	
		R4=PASS R4, R11=R12;	/* Clear AZ, hold initial SYSCON*/

		DO ___lib_RSTI UNTIL EQ;/* Setup dummy loop		*/

		FLUSH CACHE;		/* Clean off old top-of-loop	*/
		R0=START_ADDR+0x4;	/*  and replace with new	*/
		PCSTK=R0;		/*  top-of-loop value		*/

		DM(IIEP0)=R1;		/* Setup DMA to load over ldr	*/
		DM(CEP0)=R2;		/* Load internal count		*/
		DM(ECEP0)=R4;		/* Load external count		*/
		JUMP START_ADDR+0x4 (DB);/* Jump to start		*/
		DM(DMAC0)=R14;		/* Start DMA transfer		*/
		IDLE;				/* After IDLE, patch then start	*/

test_dm16_zero:	R0=R0-1;
		IF EQ JUMP (PC, dm16_zero);
		R0=R0-1;
		IF NE JUMP (PC, test_dm40_zero);

dm32_zero:
dm16_zero:	R0=R0-R0, I0=R3;
		LCNTR=R2, DO dm16_zero_loop UNTIL LCE;	
dm16_zero_loop:		DM(I0,M6)=R0;

		JUMP read_boot_info;

test_dm40_zero:	R0=R0-1;
		IF NE JUMP (PC, test_dm16_init);

dm40_zero:	PX1=0;	
		PX2=0;
		R0=R0-R0, I0=R3;
		LCNTR=R2, DO dm40_zero_loop UNTIL LCE;
dm40_zero_loop:		DM(I0,M6)=PX;

		JUMP read_boot_info;

test_dm16_init:	R0=R0-1;
		IF NE JUMP (PC, test_dm32_init);

dm16_init:	I0=R3;
		LCNTR=R2, DO dm16_init_loop UNTIL LCE;
			CALL read_PROM_word (DB);
			NOP;             /* NOP's are required for */
			NOP;             /* loop restriction       */
dm16_init_loop:		DM(I0,M6)=R3;

		JUMP read_boot_info;

test_dm32_init:	R0=R0-1;
		IF NE JUMP (PC, test_dm40_init);

dm32_init:	I0=R3;
		LCNTR=R2, DO dm32_init_loop UNTIL LCE;
			CALL read_PROM_word (DB);
			NOP;	
			NOP;
dm32_init_loop:		DM(I0,M6)=R3;

		JUMP read_boot_info;

test_dm40_init:	R0=R0-1;
		IF NE JUMP (PC, test_pm16_zero);

dm40_init:	I0=R3;
		LCNTR=R2, DO dm40_init_loop UNTIL LCE;
			CALL read_PROM_word (DB);
			NOP;
			NOP;
dm40_init_loop:		DM(I0,M6)=PX;

		JUMP read_boot_info;

test_pm16_zero:	R0=R0-1;
		IF EQ JUMP (PC, dm16_zero);
		R0=R0-1;
		IF EQ JUMP (PC, dm32_zero);

test_pm40_zero:	R0=R0-1;
		IF EQ JUMP (PC, dm40_zero);
		R0=R0-1;
		IF NE JUMP (PC, test_pm16_init);

pm48_zero:	PX1=0;
		PX2=0;
		R0=R0-R0, I8=R3;
		LCNTR=R2, DO pm40_zero_loop UNTIL LCE;	
pm40_zero_loop:		PM(I8,M14)=PX;

		JUMP read_boot_info;

test_pm16_init:	R0=R0-1;
		IF EQ JUMP (PC, dm16_init);

test_pm32_init:	R0=R0-1;
		IF EQ JUMP (PC, dm32_init);

test_pm40_init:	R0=R0-1;
		IF EQ JUMP (PC, dm40_init);

test_pm48_init:	R0=R0-1;
		IF NE JUMP read_boot_info;

pm48_init:	
		r13 = 0x20000;
		r13=r3-r13;
		if ge jump pm48_externalMemoryInit; 

		I8=R3;
		LCNTR=R2, DO pm48_init_loop UNTIL LCE;
			CALL read_PROM_word (DB);
			NOP;
			NOP;
pm48_init_loop:		PM(I8,M14)=PX;


		JUMP read_boot_info;

pm48_externalMemoryInit:	
		I0=R3;
		BTST R3 by 0x17; 		/* Check if Live address is in boot space: ie. R3=0x0008xxxxx */
		IF sz JUMP pm48_loop; 	/* jump to eprom overlay */

/**************** Eprom Overlay *******************/
prom_ovrly:		R9=DM(EIEP0);		/* copy first locxation of Live space on eprom */
				DM(I2,M6)=R9;
				R8=0x6;				/* adjust external index next location of boot info */
				R8=R2*R8(SSI);
				R9=R8+R9;
				DM(EIEP0)=R9;
				JUMP read_boot_info;		/* read boot info */

pm48_loop:
		LCNTR=R2, DO pm48_init_external_loop UNTIL LCE;
			CALL read_PROM_word (DB);
			NOP;	
			NOP;
			dm(i0,m6)=r2;
pm48_init_external_loop:		DM(I0,M6)=R3;
		JUMP read_boot_info;


read_PROM_word:	R13=DM(SYSCON);		/* Save old value of SYSCON	*/
		DM(SYSCON)=R11;		/* Set BSO bit for ROM read	*/
		DM(IIEP0)=I15;		/* Setup DMA destination address*/
		DM(CEP0)=M14;		/* Setup DMA internal length	*/
		DM(ECEP0)=R15;		/* Setup DMA external count	*/
		DM(DMAC0)=R14;		/* Start DMA 			*/
		IDLE;				/* Wait for DMA to complete	*/
		PX=PM(START_ADDR+0x4);	/* Read word from scratch	*/
		DM(SYSCON)=R13;		/* Reset SYSCON to previous	*/
		RTS (DB);
		R2=PX1;				/* Copy PX values into DREGS	*/
		R2=PASS R2, R3=PX2;	/* Test the length		*/




.ENDSEG;






